#include "stdlib.h"
#include <stdio.h>
#include <windows.h>

#pragma GCC diagnostic ignored "-Wwrite-strings"

BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    switch (fdwReason)
    {
        case DLL_PROCESS_ATTACH:
            // attach to process
            // return FALSE to fail DLL load
            break;

        case DLL_PROCESS_DETACH:
            // detach from process
            break;

        case DLL_THREAD_ATTACH:
            // attach to thread
            break;

        case DLL_THREAD_DETACH:
            // detach from thread
            break;
    }
    return TRUE; // successful
}

#define MV_UP 1
#define MV_DOWN 2
#define MV_LEFT 3
#define MV_RIGHT 4

bool ccdok = false;
bool ccdexposing = false;
int ccdw,ccdh,ccdx,ccdy,ccdb,endtick;
int demoTemperature = -12500;
char demoStatistics[50] = "T: 12.5  50%";

int GetCCDPixel(int x, int y)
{
 int v;
 v = rand() & 0x1fff;
 if ((x & 31) == 16) v += 5000;
 if ((y & 31) == 16) v += 5000;
 return v;
}

// Functions exported by the DLL

extern "C" __declspec(dllexport)
int WINAPI d_Initialize(HWND whandle)
{
 // Optional memory allocation
 return 1;
}

extern "C" __declspec(dllexport)
int WINAPI d_Finalize()
{
 // Optional clean-up
 return 1;
}

extern "C" __declspec(dllexport)
int WINAPI d_SetupCCD(int silent)    // If silent = 0 the driver should show
{                                    // a dialog window with the ccd options.
 if (silent == 0) MessageBox(0,"OK, CCD camera found","CCD Check",MB_ICONINFORMATION + MB_SYSTEMMODAL);
 ccdok = true; // The driver should check the hardware but this function
 return 1;     // must return 1. The result of the hardware check
               // must be returned by the function below:
};

extern "C" __declspec(dllexport)
int WINAPI d_GetOK()
{
  if (ccdok) return 1; else return 0;
};

extern "C" __declspec(dllexport)
int WINAPI d_Subframe(int x, int y, int w, int h, int binmode)
{
 int binning,chipx,chipy,scale;

 chipx = 500;    // Chip has 500x350 pixels
 chipy = 350;
 switch (binmode) {
  case 0: binning = 1; break;
  case 1: binning = 2; break;
  case 2: binning = 4; break;
  default: return 0;
 };
 scale = 100000;    // Fixed percentage scale factor
 ccdb = binning;
 ccdw = (chipx / binning) * w / scale;   // The driver can trunc or round
 ccdh = (chipy / binning) * h / scale;	 // as it wish. It could also ignore
 ccdx = (chipx / binning) * x / scale;	 // the subframe and binning settings.
 ccdy = (chipy / binning) * y / scale;
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_MoveFrame(int param)
{
 switch (param) {
  case MV_UP: ccdy++; break;	    // Used by the focus window of Astroart,
  case MV_DOWN: ccdy--; break;	    // a simple driver could ignore it.
  case MV_LEFT: ccdx--; break;
  case MV_RIGHT: ccdx++; break;
 };
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_GetWidth(int param)
{
 return ccdw;
};                                  // *** The most important functions ***
                                    // They control the size of the image to be
extern "C" __declspec(dllexport)    // created into Astroart.
int WINAPI d_GetHeight(int param)
{
 return ccdh;
};

extern "C" __declspec(dllexport)
int WINAPI d_BinningCount()
{
  return 3;                     // Three binning modes available: 1x1, 2x2, 4x4
};

extern "C" __declspec(dllexport)
char* WINAPI d_BinningName(int n)
{
 switch (n) {
  case 0: return "1 X 1";
  case 1: return "2 X 2";
  case 2: return "4 X 4";
  default: return NULL;
 }
};

extern "C" __declspec(dllexport)
char* WINAPI d_GetName()
{
 return "Demo CCD Camera - Control panel";
};

extern "C" __declspec(dllexport)
char* WINAPI d_GetInfo()
{
 float temp = (float)demoTemperature / 1000.0f;
 sprintf(demoStatistics,"T: %4.1f  50%%",temp);
 return demoStatistics;
};

extern "C" __declspec(dllexport)
int WINAPI d_GetTemperature()
{
 return demoTemperature;
};

extern "C" __declspec(dllexport)
int WINAPI d_StartExposure(int expo, int openshutter)
{
 ccdexposing = true;
 endtick = (int)GetTickCount() + expo;
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_IsExposing()
{
 if (!ccdexposing) return 0;
 if ((int)GetTickCount() > endtick) {
    ccdexposing = false;
    return 0;
 };
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_StartDownload()
{
 Sleep(ccdw * ccdh / 100 + 100);        // Simulates a download delay
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_ReadImage(unsigned short* pbuffer)
{
 int x,y,c;
 c = 0;
 for (y = ccdh-1; y >= 0; y--)           // Simulates an image
 for (x = 0; x <= ccdw-1; x++) {
     pbuffer[c] = GetCCDPixel((x+ccdx)*ccdb,(y+ccdy)*ccdb);
     c++;
 };
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_EndDownload(unsigned short* pbuffer)
{
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_InterruptExposure()
{
 ccdexposing = false;
 return 1;
};

extern "C" __declspec(dllexport)
int WINAPI d_IsColor()
{
 return 0;
};

enum TTelCommand
{te_present, te_name, te_initialize, te_finalize,
 te_n, te_s, te_e, te_w, te_ne, te_nw, te_se, te_sw};

void relays(int tN, int tS, int tE, int tW)
{
}

extern "C" __declspec(dllexport)
int WINAPI d_Telescope(int cmd, int t)   // This function is optional.
{
 int ok;
 char *RelaysName = "DemoCamera CCD Relays";

 // "cmd" is a number [0..11] which can be defined by the following C enum:
 // {te_present, te_name, te_initialize, te_finalize, te_n, te_s, te_e, te_w,
 // te_ne, te_nw, te_se, te_sw}

 if (t < 0) t = 1000; // t negative disabled by you.

 // t negative could be supported, it means "activate the relay,
 // keep it active forever, and exit immediately", it's used by the the
 // virtual keypad of Astroart.
 // If t is zero then the pulse is a "stop" command

 ok = 1;
 switch (cmd) {
  case te_present: ok = 1; break;      // change to ok = 0 to disable
  case te_name: ok = (int)RelaysName; break;
  case te_initialize: break;
  case te_finalize: break;
  case te_n: relays(t,0,0,0); break;
  case te_s: relays(0,t,0,0); break;
  case te_e: relays(0,0,t,0); break;
  case te_w: relays(0,0,0,t); break;
  case te_ne: relays(t,0,t,0); break;
  case te_se: relays(0,t,t,0); break;
  case te_nw: relays(t,0,0,t); break;
  case te_sw: relays(0,t,0,t); break;
  default: ok = 0;
 }
 return ok;
};


enum TFWCommand
{fw_present,fw_name,fw_initialize,fw_finalize,fw_setpos,fw_getpos};

extern "C" __declspec(dllexport)
int WINAPI d_FilterWheel(int cmd, int param)  // This function is optional.
{
 int ok;
 char *FWName = "DemoCamera Filter Wheel";
 static int lastpos;

 // "cmd" is a number [0..5] which can be defined by the following C enum:
 // {fw_present,fw_name,fw_initialize,fw_finalize,fw_setpos,fw_getpos}

 ok = 1;
 switch (cmd) {
  case fw_present: ok = 8; break;      // change to ok = 0 to disable
  case fw_name: ok = (int)FWName; break;
  case fw_initialize: lastpos = 1; break;
  case fw_finalize: break;
  case fw_setpos: lastpos = param; break;
  case fw_getpos: ok = lastpos; break;
  default: ok = 0;
 }
 return ok;
};

extern "C" __declspec(dllexport)
int WINAPI d_SetTemperature(int milliCelsius, int coolerPerc)  // This function is optional.
{
 demoTemperature = milliCelsius;
 return 1;
};





